// -*-c++-*-
//===========================================================
//=     University of Illinois at Urbana/Champaign          =
//=     Department of Computer Science                      =
//=     Dr. Dan Roth - Cognitive Computation Group          =
//=                                                         =
//=  Project: Snow                                          =
//=                                                         =
//=   Module: TargetRank.h                                  =
//=  Version: 2.1                                           =
//=  Authors: Jeff Rosen and Andrew Carlson                 =
//=     Date: xx/xx/98                                      = 
//=                                                         =
//= Comments:                                               =
//===========================================================

#ifndef __TARGETRANK_H__
#define __TARGETRANK_H__

#include "SnowTypes.h"
#include <iostream>
#include <vector>

struct TargetRank
{
    FeatureID   id;
    double      activation;

    TargetRank();
    TargetRank( FeatureID ident, double activ );
        
    bool operator==( const TargetRank& rhs ) const;
    bool operator!=( const TargetRank& rhs ) const;
    bool operator<( const TargetRank& rhs ) const;
    bool operator<=( const TargetRank& rhs ) const;
    bool operator>( const TargetRank& rhs ) const;
    bool operator>=( const TargetRank& rhs ) const;
};

inline TargetRank::TargetRank() :
        id(0),
        activation(0.0)
{
}

inline TargetRank::TargetRank( FeatureID ident, double activ ) :
        id(ident),
        activation(activ)
{
}

inline bool TargetRank::operator==( const TargetRank& rhs ) const
{
    return (id == rhs.id);
}

inline bool TargetRank::operator!=( const TargetRank& rhs ) const
{
    return !operator==(rhs);
}

inline bool TargetRank::operator<( const TargetRank& rhs ) const
{
    return (activation < rhs.activation);
}

inline bool TargetRank::operator<=( const TargetRank& rhs ) const
{
    return !operator>(rhs);
}

inline bool TargetRank::operator>( const TargetRank& rhs ) const
{
    return (activation > rhs.activation);
}

inline bool TargetRank::operator>=( const TargetRank& rhs ) const
{
    return !operator<(rhs);
}


class TargetRanking : public vector<TargetRank>
{
public:
    TargetRanking();
    TargetRanking( const TargetRanking& cr );

    // returns true if the TargetRank is inserted, or false if it
    // already existed
//    bool insert( const TargetRank& r );
//    TargetRank* find( const TargetRank& r ) const;

    void Show() const;
};

inline TargetRanking::TargetRanking()
{
}

inline TargetRanking::TargetRanking( const TargetRanking& cr )
{
    *this = cr;
}

//inline bool TargetRanking::insert( const TargetRank& r )
//{
//    return set<TargetRank, greater<TargetRank> >::insert(r).second;
//}

//inline TargetRanking::iterator
//TargetRanking::find( const TargetRanking::key_type& r ) const
//{
//   return set<TargetRank, greater<TargetRank> >::find(r);
//}

inline void TargetRanking::Show() const
{
    const_iterator it = this->begin();
    const_iterator end = this->end();

    for ( ; it != end; it++)
    {
        cout << "(" << it->id << ", " << it->activation << ")" << endl;
    }
}

#endif














